import java.awt.Color;
import java.util.Random;

/**
 * This class represents a ball that bounces around a BallCourt.
 * 
 * @author Joel Ross & YOUR_NAMES_HERE
 */
public class Ball
{
    private int _diameter; //the diameter of the ball
    private double _xPosition; //the ball's xPosition
    private double _yPosition; //the ball's yPosition
    private double _xSpeed; //the speed the ball is moving along the x-axis
    private double _ySpeed; //the speed the ball is moving along the y-axis
    private Color _color; //the color of the ball

    //More constants are defined at the bottom of the class!
    
    /**
     * Constructor for objects of class BouncingBall
     */
    public Ball()
    {
    	Random randGen = new Random();
    	
    	_diameter = 40;
        _xPosition = LEFT_BOUNDARY + randGen.nextInt(BallCourt.COURT_WIDTH - _diameter);
        _yPosition = TOP_BOUNDARY + randGen.nextInt(BallCourt.COURT_HEIGHT - _diameter);
        _xSpeed = 3.5; //or try randGen.nextDouble()*7;
        _ySpeed = 3.5; //or try randGen.nextDouble()*7;

        //TODO - Make this variable more random so not all the balls are the same color!
        _color = PRETTY_COLORS[0];
    }

    
    /**
     * Moves the ball its speed; the ball bounces off of the bounds of the BallCourt, 
     * and changes color when it does so!
     */
    public void move()
    {
        //move our speed
        _xPosition = _xPosition + _xSpeed;
        _yPosition = _yPosition + _ySpeed;
        
        //Check to see if we hit a wall
        //TODO -- CHANGE WHEN WE HIT A WALL!
        if(_xPosition < LEFT_BOUNDARY)
        {
            _xPosition = LEFT_BOUNDARY;
            _xSpeed = -1*_xSpeed;
        }
        else if(_xPosition+_diameter > RIGHT_BOUNDARY)
        {
            _xPosition = RIGHT_BOUNDARY - _diameter;
            _xSpeed = -1*_xSpeed;
        }
        if(_yPosition < TOP_BOUNDARY)
        {
            _yPosition = TOP_BOUNDARY;
            _ySpeed = -1*_ySpeed;
        }
        else if(_yPosition+_diameter > BOTTOM_BOUNDARY)
        {
            _yPosition = BOTTOM_BOUNDARY - _diameter;
            _ySpeed = -1*_ySpeed;
        }
    }
    
    /**
     * Gets the current x position of the ball
     * @return the x position of the ball
     */
    public int getX()
    {
        return (int)_xPosition;
    }

    /**
     * Gets the current y position of the ball
     * @return the y position of the ball
     */
    public int getY()
    {
        return (int)_yPosition;
    }
    
    /**
     * Gets the diamater of the ball
     * @return the diameter of the ball
     */
    public int getDiameter()
    {
        return _diameter;
    }
    
    /**
     * Gets the current color of the ball
     * @return the color of the ball as a Color object
     */
    public Color getColor()
    {
        return _color;
    }
    
    /**
     * Determines whether this ball is touching (overlaps) with the given other ball.
     * @param other the other ball to check again
     * @return whether or not the two balls are touching
     */
    public boolean isTouching(Ball other)
    {
        double distance = Math.sqrt(
            (_xPosition-other._xPosition)*(_xPosition-other._xPosition) +
            (_yPosition-other._yPosition)*(_yPosition-other._yPosition)); //pythagorean theorem
            
        return distance < (_diameter/2.0+other._diameter/2.0); //touching if distance is less than sum of the radii
    }
    
    /**
     * Adjusts our speed/direction as we bounce off another ball
     * @param other the ball we have bounced off
     */
    public void bounceOff(Ball other)
    {
        //reverse direction
        _xSpeed = -1*_xSpeed;
        _ySpeed = -1*_ySpeed;

        double distance = Math.sqrt(
            (_xPosition-other._xPosition)*(_xPosition-other._xPosition) +
            (_yPosition-other._yPosition)*(_yPosition-other._yPosition)); //pythagorean theorem

        double overlap = (this._diameter+other._diameter)/2.0 - distance;
        if(overlap > 0) //if we are still touching
        {
        	System.out.println("bounce !" + overlap);
        	//move us something approximating the distance away. This should mostly put us out of touch
            _xPosition += Math.signum(_xSpeed)*(overlap*Math.sqrt(2)/2);
            _yPosition += Math.signum(_ySpeed)*(overlap*Math.sqrt(2)/2);
        }
    }
    
    /****CONSTANTS FOR YOUR CONVENIENCE --- DO NOT MODIFY****/
    //a list of less garish colors to use when drawing the balls. Generated by Chuck Hommel
    //This is short-hand for creating an array of Color objects (a Color[])
    public static final Color[] PRETTY_COLORS = {
            new Color(255,38,38), //red
            new Color(255,168,38), //orange
            new Color(212,255,38), //yellow-green
            new Color(82,255,38), //green
            new Color(38,255,125), //green-blue
            new Color(38,255,255), //light-blue
            new Color(38,125,255), //dark blue
            new Color(82,38,255), //dark purple
            new Color(212,38,255), //light purple
            new Color(255,38,168), //magenta
        }; 

    //convenience constants for court boundaries--these make life a bit easier
    private final int TOP_BOUNDARY = BallCourt.Y_CORNER;
    private final int LEFT_BOUNDARY = BallCourt.X_CORNER;
    private final int BOTTOM_BOUNDARY = BallCourt.Y_CORNER + BallCourt.COURT_HEIGHT;
    private final int RIGHT_BOUNDARY = BallCourt.X_CORNER + BallCourt.COURT_WIDTH;
    /**** END CONSTANTS ****/

}
